/*
 *                      Yeppp! library build framework
 *
 * This file is part of Yeppp! library infrastructure and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

package info.yeppp.Build;

import info.yeppp.Build.internals.CommandLineTool;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.SystemUtils;

import java.io.File;
import java.io.IOException;
import java.util.*;

public class GnuLinker extends AbstractLinker {

	public GnuLinker() {
	}

	public GnuLinker(GnuLinker prototype) {
		super(prototype);
	}

	public BuildMessages linkExecutable(String executablePath, Collection<String> objectPaths) {
		if (binariesDirectory == null) {
			throw new IllegalStateException("Binaries directory was not assigned any value");
		}
		final BuildMessages buildMessages = new BuildMessages();

        FileUtils.deleteQuietly(new File(executablePath));
		try {
			final List<String> commandLine = composeCommandLine(executablePath, objectPaths, false, null);
			final Map<String, String> environment = composeEnvironment();
            System.out.println(StringUtils.join(commandLine, " "));
			final CommandLineTool commandLineTool = new CommandLineTool(commandLine, environment);
			for (String line : commandLineTool.getError()) {
				System.err.println(line);
			}
			for (String line : commandLineTool.getOutput()) {
				System.out.println(line);
			}
		} catch (Exception exception) {
			buildMessages.add(new BuildMessage(BuildMessage.Type.Error, executablePath, null, null, exception.getMessage()));
		}
		return buildMessages;
	}

	public BuildMessages linkDynamicLibrary(String libraryPath, Collection<String> objectPaths, String specificationPath) {
		if (binariesDirectory == null) {
			throw new IllegalStateException("Binaries directory was not assigned any value");
		}
		libraryPath = FilenameUtils.concat((new File(libraryPath)).getParent(), "lib" + FilenameUtils.getBaseName(libraryPath) + ".so");
		final BuildMessages buildMessages = new BuildMessages();

		FileUtils.deleteQuietly(new File(libraryPath));
		try {
			final List<String> commandLine = composeCommandLine(libraryPath, objectPaths, true, specificationPath);
			final Map<String, String> environment = composeEnvironment();
			System.out.println(StringUtils.join(commandLine, " "));
			final CommandLineTool commandLineTool = new CommandLineTool(commandLine, environment);
			for (String line : commandLineTool.getError()) {
				System.err.println(line);
			}
			for (String line : commandLineTool.getOutput()) {
				System.out.println(line);
			}
		} catch (Exception exception) {
			buildMessages.add(new BuildMessage(BuildMessage.Type.Error, libraryPath, null, null, exception.getMessage()));
		}
		return buildMessages;
	}

	private List<String> composeCommandLine(String outputPath, Collection<String> objectPaths, boolean isLibrary, String specificationPath) {
		final List<String> commandLine = new ArrayList<String>();
		commandLine.add("g++");
		if (!runtimeLibraryUse) {
			commandLine.add("-nostdlib");
		}
		commandLine.add("-Wl,-z,noexecstack");
		for (String libraryDirectory : libraryDirectories) {
			commandLine.add("-L" + libraryDirectory);
		}
		if (isLibrary) {
			commandLine.add("-shared");
			if (specificationPath != null) {
				specificationPath = FilenameUtils.concat((new File(specificationPath)).getParent(), "lib" + FilenameUtils.getBaseName(specificationPath) + ".so.map");
				commandLine.add("-Wl,--version-script=" + specificationPath);
			}
		}
		commandLine.add("-o" + outputPath);
		for (String objectPath : objectPaths) {
			commandLine.add(objectPath);
		}
        if (!staticLibraries.isEmpty()) {
            commandLine.add("-Wl,-Bstatic");
            for (String library : staticLibraries) {
                commandLine.add("-l" + library);
            }
        }
        if (!dynamicLibraries.isEmpty()) {
            commandLine.add("-Wl,-Bdynamic");
            for (String library : dynamicLibraries) {
                commandLine.add("-l" + library);
            }
        }
		return commandLine;
	}

	private Map<String, String> composeEnvironment() throws IOException {
		final Map<String, String> environment = new HashMap<String, String>(System.getenv());
		boolean processedIncludeVariable = false;
		for (Map.Entry<String, String> variable : environment.entrySet()) {
			if (variable.getKey().equalsIgnoreCase("LIB")) {
				String include = variable.getValue();
				include = StringUtils.join(globalLibraryDirectories, SystemUtils.PATH_SEPARATOR) +
						SystemUtils.PATH_SEPARATOR + include;
				variable.setValue(include);
				processedIncludeVariable = true;
			}
		}
		if (processedIncludeVariable == false) {
			environment.put("LIB", StringUtils.join(globalLibraryDirectories, SystemUtils.PATH_SEPARATOR));
		}
		return environment;
	}

	@Override
	public void addDefaultGlobalLibraryDirectories() {
	}

	@Override
	public Linker copy() {
		return new GnuLinker(this);
	}

}