/*
 *                      Yeppp! library implementation
 *
 * This file is part of Yeppp! library and licensed under 2-clause BSD license.
 * See library/LICENSE.txt for details.
 *
 */

#include <yepPredefines.h>
#include <yepLibrary.h>
#include <yepCore.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <assert.h>
//~ #include <core/functions.h>

YEP_INLINE static void* aligned_alloc(size_t allocation_size, size_t alignment) {
	#ifdef _MSC_VER
		// Microsoft flavor
		return _aligned_malloc(allocation_size, alignment);
	#else
		// Unix flavor
		return memalign(alignment, allocation_size);
	#endif
}

YEP_INLINE static void aligned_free(void* memory_pointer) {
	#ifdef _MSC_VER
		// Microsoft flavor
		_aligned_free(memory_pointer);
	#else
		// Unix flavor
		free(memory_pointer);
	#endif
}

void reportResults(const char* functionName, const char* cacheLevelName, const char* opsUnits, Yep64f opsScale, Yep32u opsPerElement, Yep64u elements, Yep64u ticks, Yep64u frequency, Yep64u cycles) {
	const Yep64f time = Yep64f(ticks) / Yep64f(frequency);
	const Yep64f opsPerSecond = Yep64f(elements) / time;
	const Yep64f CPE = Yep64f(cycles) / Yep64f(elements);
	printf("%s\t%s\t%.3f %s\t%.2f CPE\n", functionName, cacheLevelName, opsPerSecond / opsScale, opsUnits, CPE);
}

int main(int argc, char** argv) {
	YepStatus status;
	status = yepLibrary_Init();
	assert(status == YepStatusOk);
	
	Yep64u isaFeatures, simdFeatures, systemFeatures;
	status = yepLibrary_GetCpuIsaFeatures(&isaFeatures);
	assert(status == YepStatusOk);
	status = yepLibrary_GetCpuSimdFeatures(&simdFeatures);
	assert(status == YepStatusOk);
	status = yepLibrary_GetCpuSystemFeatures(&systemFeatures);
	assert(status == YepStatusOk);
	
	Yep32u cacheSizes[4];
	yepLibrary_GetCpuDataCacheSize(1, &cacheSizes[0]);
	yepLibrary_GetCpuDataCacheSize(2, &cacheSizes[1]);
	yepLibrary_GetCpuDataCacheSize(3, &cacheSizes[2]);
	cacheSizes[3] = 640 * 1024 * 1024;
	const char* cacheLevelName[4] = {"L1", "L2", "L3", "RAM"};
	
	Yep64u startTicks, endTicks, frequency, cpuCycles, cpuState;
	status = yepLibrary_GetTimerFrequency(&frequency);
	assert(status == YepStatusOk);

	for (Yep32u cacheLevel = 0; cacheLevel < YEP_COUNT_OF(cacheSizes); cacheLevel++) {
		Yep32u cacheSize = cacheSizes[cacheLevel];
		YepSize sampleSize = cacheSize / 4 / sizeof(Yep8u);
		if (cacheSize == 0)
			continue;

		Yep8u* a = (Yep8u*)aligned_alloc(sampleSize * sizeof(Yep8u), 16);
		Yep8u* b = (Yep8u*)aligned_alloc(sampleSize * sizeof(Yep8u), 16);
		Yep8u* c = (Yep8u*)aligned_alloc(sampleSize * sizeof(Yep8u), 16);
		
		status = yepCore_Add_V8uV8u_V8u(a, b, c, sampleSize);
		assert(status == YepStatusOk);
		
		status = yepLibrary_GetTimerTicks(&startTicks);
		assert(status == YepStatusOk);
		status = yepLibrary_GetCpuCyclesAcquire(&cpuState);
		assert(status == YepStatusOk);
		status = yepCore_Add_V8uV8u_V8u(a, b, c, sampleSize);
		assert(status == YepStatusOk);
		status = yepLibrary_GetCpuCyclesRelease(&cpuState, &cpuCycles);
		assert(status == YepStatusOk);
		status = yepLibrary_GetTimerTicks(&endTicks);
		assert(status == YepStatusOk);
		reportResults("yepCore_Add_V8uV8u_V8u", cacheLevelName[cacheLevel], "MIPS", 1.0e+6, 1, sampleSize, endTicks - startTicks, frequency, cpuCycles);

		aligned_free(a);
		aligned_free(b);
		aligned_free(c);
	}

	for (Yep32u cacheLevel = 0; cacheLevel < YEP_COUNT_OF(cacheSizes); cacheLevel++) {
		Yep32u cacheSize = cacheSizes[cacheLevel];
		YepSize sampleSize = cacheSize / 4 / sizeof(Yep32f);
		if (cacheSize == 0)
			continue;

		Yep32f* a = (Yep32f*)aligned_alloc(sampleSize * sizeof(Yep32f), sizeof(Yep32f));
		Yep32f* b = (Yep32f*)aligned_alloc(sampleSize * sizeof(Yep32f), sizeof(Yep32f));
		Yep32f* c = (Yep32f*)aligned_alloc(sampleSize * sizeof(Yep32f), sizeof(Yep32f));
		
		status = yepCore_Add_V32fV32f_V32f(a, b, c, sampleSize);
		assert(status == YepStatusOk);
		
		status = yepLibrary_GetTimerTicks(&startTicks);
		assert(status == YepStatusOk);
		status = yepLibrary_GetCpuCyclesAcquire(&cpuState);
		assert(status == YepStatusOk);
		status = yepCore_Add_V32fV32f_V32f(a, b, c, sampleSize);
		assert(status == YepStatusOk);
		status = yepLibrary_GetCpuCyclesRelease(&cpuState, &cpuCycles);
		assert(status == YepStatusOk);
		status = yepLibrary_GetTimerTicks(&endTicks);
		assert(status == YepStatusOk);
		reportResults("yepCore_Add_V32fV32f_V32f", cacheLevelName[cacheLevel], "MFLOPS", 1.0e+6, 1, sampleSize, endTicks - startTicks, frequency, cpuCycles);

		aligned_free(a);
		aligned_free(b);
		aligned_free(c);
	}

	for (Yep32u cacheLevel = 0; cacheLevel < YEP_COUNT_OF(cacheSizes); cacheLevel++) {
		Yep32u cacheSize = cacheSizes[cacheLevel];
		YepSize sampleSize = cacheSize / 4 / sizeof(Yep64f);
		if (cacheSize == 0)
			continue;

		Yep64f* a = (Yep64f*)aligned_alloc(sampleSize * sizeof(Yep64f), sizeof(Yep64f));
		Yep64f* b = (Yep64f*)aligned_alloc(sampleSize * sizeof(Yep64f), sizeof(Yep64f));
		Yep64f* c = (Yep64f*)aligned_alloc(sampleSize * sizeof(Yep64f), sizeof(Yep64f));
		
		status = yepCore_Add_V64fV64f_V64f(a, b, c, sampleSize);
		assert(status == YepStatusOk);
		
		status = yepLibrary_GetTimerTicks(&startTicks);
		assert(status == YepStatusOk);
		status = yepLibrary_GetCpuCyclesAcquire(&cpuState);
		assert(status == YepStatusOk);
		status = yepCore_Add_V64fV64f_V64f(a, b, c, sampleSize);
		assert(status == YepStatusOk);
		status = yepLibrary_GetCpuCyclesRelease(&cpuState, &cpuCycles);
		assert(status == YepStatusOk);
		status = yepLibrary_GetTimerTicks(&endTicks);
		assert(status == YepStatusOk);
		reportResults("yepCore_Add_V64fV64f_V64f", cacheLevelName[cacheLevel], "MFLOPS", 1.0e+6, 1, sampleSize, endTicks - startTicks, frequency, cpuCycles);

		aligned_free(a);
		aligned_free(b);
		aligned_free(c);
	}

}